package main

import (
	permissionservice "ark-permission/internal/server/permissionservice"
	roleservice "ark-permission/internal/server/roleservice"
	tokenservice "ark-permission/internal/server/tokenservice"
	"flag"
	"fmt"

	"ark-permission/gen_result/pb/permission"
	"ark-permission/internal/config"
	"ark-permission/internal/svc"

	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/service"
	"github.com/zeromicro/go-zero/zrpc"
	"google.golang.org/grpc"
	"google.golang.org/grpc/reflection"
)

var configFile = flag.String("f", "etc/permission.yaml", "the config file")

func main() {
	flag.Parse()

	var c config.Config
	conf.MustLoad(*configFile, &c)
	ctx := svc.NewServiceContext(c)

	s := zrpc.MustNewServer(c.RpcServerConf, func(grpcServer *grpc.Server) {
		permission.RegisterTokenServiceServer(grpcServer, tokenservice.NewTokenServiceServer(ctx))
		permission.RegisterRoleServiceServer(grpcServer, roleservice.NewRoleServiceServer(ctx))
		permission.RegisterPermissionServiceServer(grpcServer, permissionservice.NewPermissionServiceServer(ctx))

		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	})
	defer s.Stop()

	// 加入中間件
	// s.AddUnaryInterceptors(middleware.TimeoutMiddleware)

	fmt.Printf("Starting rpc server at %s...\n", c.ListenOn)
	s.Start()
}
